use std::env;
use std::fs;
use std::path::{Path, PathBuf};

use futures_util::StreamExt;
use indicatif::{ProgressBar, ProgressStyle};
use path_absolutize::Absolutize;
use reqwest::Client;
use tokio::fs::{File, OpenOptions};
use tokio::io::{AsyncSeekExt, AsyncWriteExt, SeekFrom};
use tracing::{debug, info};

use crate::util::uuid;

/// 返回一个临时目录路径，可指定是否立即创建其路径。
///
/// 该函数生成一个基于系统临时目录的新目录路径，并根据 `create` 参数决定是否创建该目录。
/// 生成的路径包含一个唯一标识符，以确保每次调用都生成不同的目录。
///
/// # 参数
/// - `create`: 一个布尔值，如果为 `true`，则创建目录。
///
/// # 返回值
/// 返回一个包含生成路径的 `Result` 类型。如果 `create` 为 `true` 且目录创建失败，则返回错误信息。
///
/// # 示例
/// ```rust
/// let temp_dir_path = temp_dir(true)?;
/// println!("Temporary directory path: {:?}", temp_dir_path);
/// ```
pub fn temp_dir(create: bool) -> crate::Result<PathBuf> {
    // 获取系统临时目录路径并添加一个唯一标识符
    let mut path = PathBuf::from(env::temp_dir());
    path.push(uuid());
    // 如果 `create` 为 `true`，则创建该目录
    if create {
        fs::create_dir(&path)?;
        debug!("create temp_dir: {}", &path.to_str().unwrap());
    }
    // 返回生成的路径
    Ok(path)
}

/// 遍历目录并返回包含所有文件路径的向量。
///
/// 该函数递归遍历给定路径下的所有文件和目录，并将其路径存储在一个向量中返回。
/// 如果 `nodir` 参数为 `true`，则不会将目录路径添加到结果中。
///
/// # 参数
/// - `path`: 要遍历的目录路径。
/// - `nodir`: 一个布尔值，如果为 `true`，则不会将目录路径添加到结果中。
///
/// # 返回值
/// 返回一个包含所有文件和（可选）目录路径的向量。
///
/// # 示例
/// ```rust
/// let files = walk_dir("/path/to/dir", true);
/// for file in files {
///     println!("{:?}", file);
/// }
/// ```
pub fn walk_dir<P: AsRef<Path>>(path: P, nodir: bool) -> Vec<PathBuf> {
    // 创建一个存储结果的向量
    let mut res = vec![];
    // 遍历给定路径下的所有条目
    for entry in std::fs::read_dir(path).expect("read_dir call failed!") {
        // 获取条目的路径
        let entry = entry.unwrap().path();
        // 如果条目是目录
        if entry.is_dir() {
            // 如果 `nodir` 为 `false`，则将目录路径添加到结果中
            if !nodir {
                res.push(entry.clone());
            }
            // 递归遍历目录并将结果添加到当前结果中
            res.append(&mut walk_dir(entry, nodir));
        } else {
            // 如果条目是文件，则将其路径添加到结果中
            res.push(entry);
        }
    }
    // 返回包含所有文件和（可选）目录路径的向量
    res
}

/// 设置或清除下载进程的环境变量 `CCUTIL_DOWNLOAD_PROCESS`。在使用 `download` 和 `async_download`
/// 函数时，会根据是否设置此变量来显示或隐藏下载进度。
///
/// # 参数
///
/// - `with_process`: 一个布尔值，指示是否要设置下载进程环境变量。
///
/// # 示例
///
/// ```rust
/// // 设置下载进程环境变量
/// set_download_process(true);
///
/// // 清除下载进程环境变量
/// set_download_process(false);
/// ```
pub fn set_download_process(with_process: bool) {
    if with_process {
        // 设置环境变量为字符串 "1"
        env::set_var("CCUTIL_DOWNLOAD_PROCESS", "1");
    } else {
        // 清除环境变量
        env::remove_var("CCUTIL_DOWNLOAD_PROCESS");
    }
}

/// 从给定的 URL 下载文件并将其保存到指定的目标路径。
///
/// 该函数初始化一个异步运行时，并阻塞在异步下载函数上，直到下载完成。
///
/// # 参数
/// - `url`: 要下载文件的 URL。
/// - `target`: 一个可选参数，指定下载文件保存的路径。如果为 `None`，函数会自动确定保存路径。
///
/// # 返回值
/// 该函数返回一个 `Result` 类型，如果下载成功则为 `Ok(())`，如果发生错误则返回错误信息。
///
/// # 错误
/// 如果异步运行时初始化失败或者下载失败，该函数会返回一个错误。
///
/// # 示例
/// ```
/// let url = "https://example.com/file.zip";
/// let target_path = Some("/path/to/save/file.zip");
/// download(url, target_path)?;
/// ```
///
/// # 备注
/// `crate::async_runtime` 函数是一个辅助函数，用于返回一个异步运行时的引用。`async_download` 函数是执行实际异步下载过程的实现。
pub fn download<P: AsRef<Path>>(url: String, target: Option<P>) -> crate::Result<()> {
    // 使用库中的辅助函数初始化异步运行时。
    let rt = crate::async_runtime()?;
    // 阻塞在异步下载函数上，直到下载完成。
    rt.block_on(async_download(url, target))
}

/// 异步下载文件并保存到指定的目标路径。
///
/// 该函数使用异步 HTTP 客户端从指定的 URL 下载文件，并将其保存到指定的目标路径。
/// 如果目标路径为空，则使用 URL 的文件名作为本地文件名。
/// 支持断点续传功能。
///
/// # 参数
/// - `url`: 要下载文件的 URL。
/// - `target`: 一个可选参数，指定下载文件保存的路径。如果为空，则使用 URL 的文件名。
///
/// # 返回值
/// 该函数返回一个 `Result` 类型，如果下载成功则为 `Ok(())`，如果发生错误则返回错误信息。
///
/// # 错误
/// 如果下载文件的父目录不存在、无法创建或写入文件、或者下载过程中出现任何问题，该函数会返回一个错误。
///
/// # 示例
/// ```rust
/// let url = "https://example.com/file.zip";
/// let target_path = Some("/path/to/save/file.zip");
/// async_download(url, target_path).await?;
/// ```
///
/// # 备注
/// 使用 `Client` 进行 HTTP 请求，`OpenOptions` 和 `File` 进行文件操作，支持断点续传。
pub async fn async_download<P: AsRef<Path>>(url: String, target: Option<P>) -> crate::Result<()> {
    debug!("Download url: {}", url);
    // 创建一个新的 HTTP 客户端
    let client = Client::new();
    // 发送 GET 请求获取响应
    let response = client.get(url.clone()).send().await?;

    // 检查连接状态，失败会返回错误
    if !response.status().is_success() {
        return Err(format!("下载 {} 失败：{}", url, response.status()).into());
    }

    // 确定下载文件的路径
    let download_file = match target {
        Some(file) => {
            let file = abs_path(Path::new(file.as_ref()))?;
            // 检查下载文件的父目录是否存在
            if !parent_exists(&file) {
                return Err("下载文件的父级目录不存在！".into());
            }
            file
        }
        None => {
            // 如果未指定目标路径，则使用 URL 的文件名
            let _remote = Path::new(&url);
            let _local = match _remote.file_name() {
                Some(file) => file,
                None => return Err("无效的文件名！".into()),
            };
            debug!("Get raw name by url: {:?}", _local);
            abs_path(Path::new(_local))?
        }
    };
    debug!("Download file: {:?}", download_file);

    // 打开或创建文件
    let mut file = if download_file.exists() {
        info!("File exists, continue downloading.");
        OpenOptions::new()
            .read(true)
            .write(true)
            .open(&download_file)
            .await?
    } else {
        File::create(&download_file).await?
    };

    // 获取当前文件的大小和总大小
    let mut current_size = file.metadata().await?.len();
    let total_size = response.content_length().unwrap();
    // 如果文件已经完全下载，则返回
    if current_size == total_size {
        info!("File already downloaded: {:?}.", download_file);
        return Ok(());
    }

    // 如果文件部分下载过，则进行断点续传
    let mut request = client.get(url);
    if current_size > 0 {
        request = request.header("Range", format!("bytes={}-", current_size));
        file.seek(SeekFrom::Start(current_size)).await?;
    }

    // 发送请求并获取响应
    let response = request.send().await?;
    let mut stream = response.bytes_stream();

    // 设置下载进度条
    let pb = ProgressBar::new(total_size);
    let with_process = match env::var("CCUTIL_DOWNLOAD_PROCESS") {
        Ok(_) => {
            pb.set_style(
                ProgressStyle::default_bar()
                    .template("{spinner:.green} [{elapsed_precise}] [{wide_bar:.cyan/blue}] {bytes}/{total_bytes} ({bytes_per_sec}, {eta_precise})").unwrap()
                    .progress_chars("#>-"),
            );
            true
        }
        Err(_) => false,
    };

    // 读取响应数据并写入文件
    while let Some(chunk) = stream.next().await {
        let chunk = chunk?;
        file.write_all(&chunk).await?;
        current_size += chunk.len() as u64;

        // 更新下载进度
        if with_process {
            pb.set_position(current_size);
        }
    }

    Ok(())
}

/// 检查给定路径的父目录是否存在。
///
/// 该函数获取路径的父目录并检查该目录是否存在。
///
/// # 参数
/// - `path`: 要检查的路径。
///
/// # 返回值
/// 如果父目录存在则返回 `true`，否则返回 `false`。
///
/// # 示例
/// ```
/// use std::path::Path;
/// let path = Path::new("/some/directory/file.txt");
/// assert_eq!(parent_exists(&path), true);
/// ```
pub fn parent_exists(path: &Path) -> bool {
    // 获取路径的父目录，并检查该目录是否存在。
    match path.parent() {
        Some(parent) => parent.exists(),
        None => false,
    }
}

/// 获取给定路径的绝对路径。
///
/// 该函数接收一个路径并返回其绝对路径。
///
/// # 参数
/// - `path`: 要转换为绝对路径的相对路径或绝对路径。
///
/// # 返回值
/// 该函数返回一个 `Result` 类型，如果转换成功则为 `Ok(PathBuf)`，如果发生错误则返回错误信息。
///
/// # 错误
/// 如果路径无法转换为绝对路径，该函数会返回一个错误。
///
/// # 示例
/// ```rust
/// use std::path::Path;
/// let relative_path = Path::new("some/relative/path");
/// let absolute_path = abs_path(&relative_path)?;
/// println!("Absolute path: {:?}", absolute_path);
/// ```
pub fn abs_path(path: &Path) -> crate::Result<PathBuf> {
    // 将路径转换为绝对路径，并将其转换为 PathBuf 类型
    let abs = path.absolutize()?.to_path_buf();
    Ok(abs)
}
